#-----------------------------------------------------------------------------
# Extractor for CR2
# Copyright (c) 2003, David G. Drumright (ockham)
# Trial version, Aug. 21, 2003
# Specify the CR2.  Finds all textures and OBJ files,
# and moves them to a 'dump' folder where
# you can zip them or do whatever.
#-----------------------------------------------------------------------------

from Tkinter import *

import os
import glob
import string
import shutil
import poser

BasePathUncut = poser.AppLocation()  # So we can strip off the full paths given by tkFileDialog
BasePath = BasePathUncut[:(-len('poser.exe'))]
CharPath = BasePath + 'Runtime/Libraries/character/'

#-----------------------------------------------------------------------------
# Global vars
#-----------------------------------------------------------------------------

global Go
global CR2GivenName
Go = 0
CR2GivenName=""
global SearchList
SearchList=[]
# Each entry is tuple of filename and dir-where-found

#-----------------------------------------------------------------------------

def WalkGlobber(arg,dirname,names):
    global SearchList
    for onename in names:
        Upname = string.upper(onename)
        Uparg  = string.upper(arg)
        if string.count(Upname,Uparg) and not string.count(dirname,'EXTRACTED'):
            SearchList.append((onename,dirname))

#-----------------------------------------------------------------------------

def Action():
    global CR2GivenName
    global SearchList
    # Start by stripping off the extension if present.

    CR2GivenName = string.upper(string.strip(CR2GivenName))
    CR2GivenName = string.replace(CR2GivenName,':','/') # De-mac
    CR2GivenName = string.replace(CR2GivenName,'\\','/') # De-dos
    if string.count(CR2GivenName,'.CRZ'):
        raise "Can't handle compressed files."
    elif string.count(CR2GivenName,'.CR2'):
        CR2FullName = CharPath  + CR2GivenName
    else:
        CR2FullName = CharPath  + CR2GivenName + '.CR2'
    CR2BaseName,dummy = os.path.splitext(os.path.basename(CR2FullName))

    # Locate the CR2
    if not os.path.exists(CR2FullName):
        raise('%s not present, or name badly formed.' % CR2FullName)

    DumpFolderName = BasePath + '/Runtime/' + 'EXTRACTED_' + CR2BaseName
    DumpFolderNorm = os.path.normpath(DumpFolderName)
    # Run a try loop to generate new folders.  Because the same-named
    # CR2 could exist in different places, and may in fact be different
    # figures, we don't want to just err-out on duplication.
    Suf = 0
    while 1:
        if not os.path.exists(DumpFolderNorm):
            break
        else:
            Suf=Suf+1
            # Set up a new dump folder.
            DumpFolderSuf=DumpFolderName+'_'+str(Suf)
            DumpFolderNorm = os.path.normpath(DumpFolderSuf)

    os.mkdir(DumpFolderNorm)

    # Create our log file to make notes on source of odd files if any.
    LogName = os.path.normpath(DumpFolderNorm+'/Extracted.log')
    fpLog = open(LogName,"wt")


    # Before opening it, copy it into the dump folder.
    CR2Dst = os.path.normpath(DumpFolderNorm + '/' + os.path.basename(CR2FullName))
    shutil.copyfile(CR2FullName,CR2Dst)

    # Open the CR2 and locate all Textures and Objects.  Use dicts
    # here to avoid duplication.  The key is the pathname given in the
    # CR2; the value is just a 1.  (Counterintuitive but takes advantage
    # of the property of Python dictionary)
    try:
        fpCR2 = open(CR2FullName,"rt")
    except:
        raise "CR2 exists but can't be opened.\n Full path: %s" % CR2FullName
    # Run through the CR2.  Each time we locate a texture or OBJ line, add it to
    # the appropriate dict.
    TexList = {}
    ObjList = {}
    while 1:
       st = fpCR2.readline()
       if (st == None) or (st ==''):
           break
       if string.count(st,'textureMap') or string.count(st,'bumpMap') or string.count(st,'reflectionMap') or string.count(st,'transparencyMap'):
           # Skip NO_MAP and GetStringRes entirely
           if string.count(st,'NO_MAP'):
               continue
           if string.count(st,'GetStringRes'):
               continue
           # First get rid of newline and internal quotes if any
           st = string.strip(st)
           st = string.replace(st,'"','')
           st = string.replace(st,':','/') # De-mac
           Parts = string.split(st,' ',1)
           Remainder = Parts[1] # Remainder after textureMap
           TexList[Remainder] = 1
       elif string.count(st,'figureResFile'):
           if string.count(st,'GetStringRes'):
               continue
           # First get rid of newline and internal quotes if any
           st = string.strip(st)
           st = string.replace(st,'"','')
           st = string.replace(st,':','/') # De-mac
           Parts = string.split(st,' ',1)
           Remainder = Parts[1] # Remainder after figureResFile
           ObjList[Remainder] = 1
    fpCR2.close() # Done reading the CR2

    # Now go through these condensed lists and find the actual files if we can.
    # Seek everywhere, but mark the files if not found where they should be.
    fpLog.write("Textures taken from \n%s\n\n" % CR2FullName)
    for OneTexListed in TexList.keys():
        OneTexBaseName = os.path.basename(OneTexListed)
        OneTexDestPath = os.path.normpath(DumpFolderNorm + '/' + OneTexBaseName)
        # First form up the absolute path.
        if not os.path.isabs(OneTexListed):
            OneTexFullPath = os.path.normpath(BasePath + 'Runtime/Textures' + OneTexListed) # Assume Poser's way of forming this name
        else:
            OneTexFullPath = os.path.normpath(OneTexListed)

        SearchList = []
        os.path.walk('.',WalkGlobber,OneTexBaseName)
        if not SearchList: # Not found at all.
            fpLog.write("%s NOT FOUND\n" % OneTexBaseName)
        else:
            SufVal = 0
            for (OneFoundTex,OneFoundDir) in SearchList: # These should be already abspathed
                # If we go beyond one, give a suffix.
                OneTexSrc = os.path.abspath(OneFoundDir + '/' + OneFoundTex)
                OneTexDst = os.path.normpath(DumpFolderNorm + '/' + OneTexBaseName)
                if SufVal > 0:
                   OneTexDst = OneTexDst + '_' + str(SufVal)
                shutil.copyfile(OneTexSrc,OneTexDst)
                fpLog.write("%s found in %s\n" % (OneFoundTex,OneFoundDir))
                SufVal = SufVal + 1

    fpLog.write("\n-----\nObjects taken from \n%s\n\n" % CR2FullName)
    for OneObjListed in ObjList.keys():
        OneObjBaseName = os.path.basename(OneObjListed)
        OneObjDestPath = os.path.normpath(DumpFolderNorm + '/' + OneObjBaseName)
        # First form up the absolute path.
        if not os.path.isabs(OneObjListed):
            OneObjFullPath = os.path.normpath(BasePath + 'Runtime/Geometries' + OneObjListed) # Assume Poser's way of forming this name
        else:
            OneObjFullPath = os.path.normpath(OneObjListed)

        SearchList = []
        os.path.walk('.',WalkGlobber,OneObjBaseName)
        if not SearchList: # Not found at all.
            fpLog.write("%s NOT FOUND\n" % OneObjBaseName)
        else:
            SufVal = 0
            for (OneFoundObj,OneFoundDir) in SearchList: # These should be already abspathed
                # If we go beyond one, give a suffix.
                OneObjSrc = os.path.abspath(OneFoundDir + '/' + OneFoundObj)
                OneObjDst = os.path.normpath(DumpFolderNorm + '/' + OneObjBaseName)
                if SufVal > 0:
                   OneObjDst = OneObjDst + '_' + str(SufVal)
                shutil.copyfile(OneObjSrc,OneObjDst)
                fpLog.write("%s found in %s\n" % (OneFoundObj,OneFoundDir))
                SufVal = SufVal + 1

    fpLog.close()

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        global Go
        Go = 0
        self.master = master
        master.title('Extractor for CR2 files')

        self.Label1 = Label(self.master,text='Fill in subfolder and CR2 file.').grid(row=0, column=0)
        self.Label2 = Label(self.master,text='Runtime/libraries/character/').grid(row=1, column=0)
        self.Entry = Entry(self.master,width=30)
        self.Entry.grid(row=1,column=1)

        # Go/nogo buttons.
        self.buttonGo = Button(self.master, text='Go', command=self.handleGo)
        self.buttonGo.grid(row=3, column=0)

        self.buttonCancel = Button(self.master, text='Cancel', command=self.handleCancel)
        self.buttonCancel.grid(row=3, column=2)

    # - - - - - - - - - - - - - - - - - -

    def handleGo(self):
        global Go
        global CR2GivenName
        Go = 1
        CR2GivenName = self.Entry.get()
        self.master.destroy()
        self.master.quit()

    # - - - - - - - - - - - - - - - - - -

    def handleCancel(self):
        global Go
        Go = 0
        self.master.destroy()
        self.master.quit()

#------------------------------------------------------------
#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
root.mainloop()


#------------------------------------------------------------
# End TK loop.

#-----------------------------------------------------------------------------
# Real actions here:
#-----------------------------------------------------------------------------

if Go:
    Action()

#-----------------------------------------------------------------------------
