#-----------------------------------------------------------------------------
# Orphan OBJ finder.
# Copyright (c) 2003, David G. Drumright (ockham)
# 11/09/2003
#-----------------------------------------------------------------------------

from Tkinter import *
root = Tk()

import os
import glob
import string

import poser

BasePathUncut=poser.AppLocation()  # So we can strip off the full paths given by tkFileDialog
BasePath=BasePathUncut[:(-len('poser.exe'))]
SourcePath = BasePath + '/Runtime/Libraries/'  # Look for everything in Props, Figs, Poses.
GeomPath   = BasePath + '/Runtime/Geometries/'

#-----------------------------------------------------------------------------
# Global vars
#-----------------------------------------------------------------------------

PosersOwnNames=[
'bizwomHi',
'brainstemHi',
'Camera',
'CasBoy',
'catHi',
'catHi',
'childCasHi',
'childCasLo',
'childHandsHi',
'childHandsLo',
'childNudeHi',
'childNudeLo',
'childStick',
'deform',
'dogHi',
'dolphHi',
'femaleCasHi',
'femaleCasLo',
'femaleHandsHi',
'femaleHandsLo',
'femaleHeadOnly',
'femaleNudeHi',
'femaleNudeLo',
'femaleOrig',
'femaleSkeleton',
'femaleStick',
'femaleSuitHi',
'femaleSuitLo',
'fish',
'hair',
'hardHi',
'helixHi',
'horseHi',
'lights',
'maleCasHi',
'maleCasLo',
'maleHandHiZ',
'maleHandsHi',
'maleHandsLo',
'maleHeadOnly',
'maleNudeHi',
'maleNudeLo',
'maleOrig',
'maleSkeleton',
'maleStick',
'maleSuitHi',
'maleSuitLo',
'manikin',
'newCasualFemale',
'newFemaleNudeHi',
'newMaleBizHi',
'newMaleCasHi',
'newMaleNudeHi',
'nudeBoy',
'NudeInfant',
'P4NudeMan',
'P4NudeWoman',
'P4NudeWoman',
'P5CasualBoy',
'P5CasualGirl',
'P5CasualMan',
'P5CasualWoman',
'P5Clothes',
'P5NudeBoy',
'P5NudeGirl',
'P5NudeMan',
'P5NudeWoman',
'P5Props',
'Poser 4 Lo',
'Poser4Clothes',
'Poser4Figures',
'Poser4Hair',
'ProPack',
'props',
'raptorHi',
'RSnake',
'ZygoteAnimals']

#-----------------------------------------------------------------------------

global UsedObjs
UsedObjs={}  # Dict so we can avoid duplication.  Keys are the basenames, values are fullnames
global FoundObjs
FoundObjs={}  # Dict so we can avoid duplication.  Keys are the basenames, values are fullnames
global app # just so we can define funcs earlier
app=0

#-----------------------------------------------------------------------------

def ShowStatus(s):
    global app
    if app:
        app.NameEntry.delete(0,END)
        app.NameEntry.insert(0,s)
        app.NameEntry.update_idletasks()
#-----------------------------------------------------------------------------

def ReadSourceInDir(pat,dirname,names):
    # Written for CR2, but in this case (unlike the object) the CR2
    # and PP2 and mat-pose Pz2 use same lines, so we use same routine for all.
    global UsedObjs

    for OneBaseName in names:
        # Open the CR2/PP2 and locate all Objects.  Use dicts
        # here to avoid duplication.  The key is the pathname given in the
        # CR2; the value is the CR2's filename.
        UpName=string.upper(OneBaseName)
        UpPat =string.upper(pat)
        if not string.count(UpName,UpPat): continue

        ShowStatus(OneBaseName)
        FullPath = os.path.normpath(dirname + '/' + OneBaseName)
        if not os.path.isfile(FullPath): continue  # this excludes the names that are actually dirs
        try:
            fpCR2 = open(FullPath,"rt")
        except:
            raise "Source exists but can't be opened.\n Full path: %s" % FullPath

        # Run through the CR2.  Each time we locate an obj line, add it to the dict.
        # In alternategeom, we have a semi-arbitrary number
        # objFile 1043 :Runtime:Geometries:LadyLittlefox:Koshini:RoundREye.obj
        # Usual in CR2 is
        # figureResFile :Runtime:Geometries:props:ball.obj
        # Usual in PP2 is
        # objFileGeom 0 0 :Runtime:Geometries:props:ball.obj
        while 1:
            st = fpCR2.readline()
            if (st == None) or (st ==''):         break
            elif string.count(st,'GetStringRes'): continue
            elif string.count(st,'NO_MAP'):       continue

            st = string.strip(st) # Get rid of indent at start and newline at end
            st = string.replace(st,'"',' ') # Make lines with and without quotes work same way
            st = string.replace(st,':','/') # Helping normpath

            if   string.count(st,'figureResFile '):  # usual in CR2
                Parts = string.split(st,' ',1) # Ignore spaces inside right part
                if Parts:
                    FullName=os.path.normpath(string.upper(Parts[1]))
                    BaseName=os.path.basename(FullName)
                    UsedObjs[BaseName] = FullName

            elif string.count(st,'objFile '):  # alt geom
                Parts = string.split(st,' ',2) # Jump across the arbitrary number
                if Parts:
                    FullName=os.path.normpath(string.upper(Parts[2]))
                    BaseName=os.path.basename(FullName)
                    UsedObjs[BaseName] = FullName

            elif string.count(st,'objFileGeom '):  # usual in PP2
                Parts = string.split(st,' ',3) # Jump across the two zeroes
                if Parts:
                    FullName=os.path.normpath(string.upper(Parts[3]))
                    BaseName=os.path.basename(FullName)
                    UsedObjs[BaseName] = FullName


        fpCR2.close() # Done reading one CR2

#-----------------------------------------------------------------------------

def ReadObjsInDir(pat,dirname,names):
    global FoundObjs
    for OneOwnName in PosersOwnNames:
        if string.count(dirname,OneOwnName): return
    for OneBaseName in names:
        FullName = os.path.normpath(dirname + '/' + OneBaseName)
        UpName = string.upper(FullName)
        if not string.count(UpName,'.OBJ'): continue
        BaseName = os.path.basename(UpName)
        FoundObjs[BaseName] = UpName

#-----------------------------------------------------------------------------

def WriteLog():
    # Create our log file and clear the clipboard
    LogName = os.path.normpath(GeomPath+'/OrphanObjects.log')
    fpLog = open(LogName,"wt")
    app.NameEntry.clipboard_clear()

    # Handle the wrong-place and not-exist separately.

    St = "--- ORPHAN OBJECTS ---\n\n"
    fpLog.write(St)
    app.NameEntry.clipboard_append(St)
    St = "---- Objects not used anywhere:\n\n"
    fpLog.write(St)
    app.NameEntry.clipboard_append(St)

    for BaseName in FoundObjs.keys():
        FullName = FoundObjs[BaseName]
        # Handle the not-exist separately.
        if BaseName not in UsedObjs.keys():
            St = "%s\n" % FullName
            fpLog.write(St)
            app.NameEntry.clipboard_append(St)


    St = "\n\n\n---- Objects in the wrong place:\n\n"
    fpLog.write(St)
    app.NameEntry.clipboard_append(St)

    for BaseName in FoundObjs.keys():
        FullName = FoundObjs[BaseName]
        # If the basename matches, but fullname (incl path) doesn't match, OBJ is in wrong place.
        if BaseName in UsedObjs.keys() and FullName not in UsedObjs.values():
            St = "%s\n" % FullName
            fpLog.write(St)
            app.NameEntry.clipboard_append(St)
    fpLog.close()

#-----------------------------------------------------------------------------
#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        global Go
        Go = 0
        self.master = master
        master.title('Orphan Object finder')

        self.NameEntry = Entry(self.master,width=30)
        self.NameEntry.grid(row=1,column=1)
        self.NameEntry.insert(END,'Hit Go to start')
        self.NameEntry.clipboard_clear()

        # Go/nogo buttons.
        self.buttonGo = Button(self.master, text='Go', command=self.handleGo)
        self.buttonGo.grid(row=3, column=0)

        self.buttonCancel = Button(self.master, text='Cancel', command=self.handleCancel)
        self.buttonCancel.grid(row=3, column=2)

    # - - - - - - - - - - - - - - - - - -

    def LocateFiles(self):
        # Main loop is a path.walk.  The inner function of the walk
        # reads the CR2, finds each figureResFile, and appends the
        # basename to the UsedObjs dict.

        os.path.walk(SourcePath,ReadSourceInDir,'.CR2')
        os.path.walk(SourcePath,ReadSourceInDir,'.PP2')
        os.path.walk(SourcePath,ReadSourceInDir,'.HR2')
        os.path.walk(SourcePath,ReadSourceInDir,'.HD2')
        os.path.walk(SourcePath,ReadSourceInDir,'.PZ2')
        os.path.walk(SourcePath,ReadSourceInDir,'.FC2')
        os.path.walk(SourcePath,ReadSourceInDir,'.CM2')
        # We now have a list of USED objects.

        # Walk through Objects.  For each OBJ we
        # find, get its basename and see if it's in the UsedObjs dict.
        # If not there, write its complete path to the log file.

        os.path.walk(GeomPath,ReadObjsInDir,None)
        WriteLog()

    # - - - - - - - - - - - - - - - - - -

    def handleGo(self):
        ShowStatus("Go drink some coffee, I'm working!.....")
        self.LocateFiles()
        ShowStatus("Done!")
        self.master.destroy()
        self.master.quit()

    # - - - - - - - - - - - - - - - - - -

    def handleCancel(self):
        self.master.destroy()
        self.master.quit()

#------------------------------------------------------------
#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

app = App(root, '')
root.mainloop()


#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
