#-----------------------------------------------------------------------------
# Orphan TEX finder.
# Copyright (c) 2003, David G. Drumright (ockham)
# 11/09/2003
# Looks through all the Textures folders, and finds textures (reflection,
# bump, etc) that are not used by any CR2 or PP2 in the same
# Runtime realm.  Writes a list of the found files, in the Textures folder,
# and also writes same list to clipboard.
# Built to run free-standing if necessary, but must be
# installed in the Poser directory to make that work.
# Special note: There are a lot of default textures that
# shouldn't be deleted.  I've tried to skip a few places
# where these are held, but probably haven't found all of them.
# So I'm assuming that you will recognize these defaults
# if any show up in the list.


#-----------------------------------------------------------------------------

from Tkinter import *
root = Tk()

import os
import glob
import string

import poser

BasePathUncut=poser.AppLocation()  # So we can strip off the full paths given by tkFileDialog
BasePath=BasePathUncut[:(-len('poser.exe'))]
SourcePath = BasePath + '/Runtime/Libraries/'  # Look for everything in Props, Figs, Poses.
TexPath = BasePath + '/Runtime/Textures/'
RefPath = BasePath + '/Runtime/Reflection Maps/'

#-----------------------------------------------------------------------------
# Global vars
#-----------------------------------------------------------------------------
StringResNames =[
'ANGELFISHMAP',
'BIZ MAN TEXTURE',
'BIZ WOMAN TEXTURE',
'CASUAL CHILD TEXTURE',
'CASUAL MAN TEXTURE',
'CASUAL WOMAN TEXTURE',
'CAT TEXTURE',
'CHILD CASUAL TEXTURE',
'CHILD HAIR TEXTURE',
'CHILD NUDE TEXTURE',
'CLASS-ROOM-FLOOR',
'DOG TEXTURE',
'DOLPHIN TEXTURE',
'FEMALE BUSINESS TEXTURE',
'FEMALE CASUAL TEXTURE',
'FEMALE HAIR 1 TEXTURE',
'FEMALE HAIR 2 TEXTURE',
'FEMALE HAIR 3 TEXTURE',
'FEMALE HAIR 4 TEXTURE',
'FEMALE HAIR 5 TEXTURE',
'FEMALE MUSCLE BUMP',
'FEMALE MUSCLE TEXTURE',
'FEMALE NUDE TEXTURE',
'FROGMAP',
'FROGMAP1',
'GOLDREFL',
'GROUND DEFAULT TEXTURE',
'GROUND DEFAULT BUMP',
'HAND TEXTURE',
'HORSE TEXTURE',
'LIONMAP',
'MALE BUSINESS TEXTURE',
'MALE CASUAL TEXTURE',
'MALE HAIR 1 TEXTURE',
'MALE HAIR 2 TEXTURE',
'MALE HAIR 3 TEXTURE',
'MALE HAIR 4 TEXTURE',
'MALE HAIR 5 TEXTURE',
'MALE MUSCLE BUMP',
'MALE MUSCLE TEXTURE',
'MALE NUDE TEXTURE',
'NUDE CHILD TEXTURE',
'NUDE MAN TEXTURE',
'NUDE WOMAN TEXTURE',
'P4 MAN BUMP',
'P4 MAN TEXTURE',
'P4 MAN TEXTURE2',
'P4 WOMAN BUMP',
'P4 WOMAN TEXTURE',
'P4 WOMAN TEXTURE2',
'P4CASBOY',
'P4CASGIRL',
'P4INFANT',
'P4NUDBOY',
'P4NUDGRL',
'RAPTOR TEXTURE',
'SILVERREFL',
'SNAKELO',
'WOLFMAP'
]

global UsedTexs
UsedTexs={}  # Dict so we can avoid duplication.  Keys are the basenames, values are fullnames
global FoundTexs
FoundTexs={}  # Dict so we can avoid duplication.  Keys are the basenames, values are fullnames
global app # just so we can define funcs earlier
app=0

PossExt=['.BMP','.BUM','.BUMP','.PCT','.PICT','.TIF','.TIFF','.JPG','JPEG']
# Note that PNG is left out because that would find thumbnails.
# Any real images in PNG form WILL show up here, maybe falsely.

#-----------------------------------------------------------------------------
def ShowStatus(s):
    global app
    if app:
        app.NameEntry.delete(0,END)
        app.NameEntry.insert(0,s)
        app.NameEntry.update_idletasks()
#-----------------------------------------------------------------------------

def ReadSourceInDir(pat,dirname,names):
    # Written for CR2, but in this case (unlike the object) the CR2
    # and PP2 and mat-pose Pz2 use same lines, so we use same routine for all.
    global UsedTexs

    for OneBaseName in names:
        # Open the CR2/PP2 and locate all Textures.  Use dicts
        # here to avoid duplication.  The key is the pathname given in the
        # CR2; the value is the CR2's filename.
        UpName=string.upper(OneBaseName)
        UpPat =string.upper(pat)
        if not string.count(UpName,UpPat): continue

        ShowStatus(OneBaseName)
        FullPath = os.path.normpath(dirname + '/' + OneBaseName)
        if not os.path.isfile(FullPath): continue  # this excludes the names that are actually dirs
        try:
            fpCR2 = open(FullPath,"rt")
        except:
            raise "Source exists but can't be opened.\n Full path: %s" % FullPath

        # Run through the CR2.  Each time we locate a tex line, add it to the dict,
        # with key=full filenam, value=type of map.
        TexLines=['textureMap','bumpMap','reflectionMap','transparencyMap']
        while 1:
            st = fpCR2.readline()
            if (st == None) or (st ==''):         break
            elif string.count(st,'GetStringRes'): continue
            elif string.count(st,'NO_MAP'):       continue

            st = string.strip(st) # Get rid of indent at start and newline at end
            st = string.replace(st,'"',' ') # Make lines with and without quotes work same way
            st = string.replace(st,':','/') # Helping normpath
            Parts = string.split(st,' ',1) # Separate the keyword from the file
            if Parts[0] in TexLines:
                FullName=os.path.normpath(string.upper(Parts[1]))
                BaseName=os.path.basename(FullName)
                UsedTexs[BaseName] = FullName

        fpCR2.close() # Done reading one CR2

#-----------------------------------------------------------------------------

def ReadTexsInDir(pat,dirname,names):
    global FoundTexs

    # Leave out the standard stuff:
    if   string.count(dirname,'Poser 3 Textures'): return
    elif string.count(dirname,'Poser 4 Textures'): return
    elif string.count(dirname,'Poser 5 Textures'): return
    elif string.count(dirname,'ProPack'): return
    elif string.count(dirname,'faceroom'): return

    for OneBaseName in names:
        FullName = os.path.normpath(dirname + '/' + OneBaseName)
        UpName = string.upper(FullName)
        GoodType=0
        for OnePat in PossExt: # Easier to handle than a long chain of string.count ORs.
            if string.count(UpName,OnePat):
                GoodType=1
                break
        if not GoodType: continue

        # Filter out the poser basic names, some of which were in those dirs anyway.
        SRName=0
        for OneStringRes in StringResNames:
            if string.count(UpName,OneStringRes):
                SRName=1
                break
        if SRName: continue

        BaseName = os.path.basename(UpName)
        FoundTexs[BaseName] = FullName

#-----------------------------------------------------------------------------

def WriteLog():
    # Create our log file and clear the clipboard
    LogName = os.path.normpath(TexPath+'/OrphanTextures.log')
    fpLog = open(LogName,"wt")
    app.NameEntry.clipboard_clear()

    # Handle the wrong-place and not-exist separately.

    St = "--- ORPHAN TEXTURES ---\n\n"
    fpLog.write(St)
    app.NameEntry.clipboard_append(St)
    St = "---- Textures not used anywhere:\n\n"
    fpLog.write(St)
    app.NameEntry.clipboard_append(St)

    for BaseName in FoundTexs.keys():
        FullName = FoundTexs[BaseName]
        # Handle the not-exist separately.
        if BaseName not in UsedTexs.keys():
            St = "%s\n" % FullName
            fpLog.write(St)
            app.NameEntry.clipboard_append(St)


    St = "\n\n\n---- Textures in the wrong place:\n\n"
    fpLog.write(St)
    app.NameEntry.clipboard_append(St)

    for BaseName in FoundTexs.keys():
        FullName = FoundTexs[BaseName]
        # If the basename matches, but fullname (incl path) doesn't match, tex is in wrong place.
        if BaseName in UsedTexs.keys() and FullName not in UsedTexs.values():
            St = "%s\n" % FullName
            fpLog.write(St)
            app.NameEntry.clipboard_append(St)
    fpLog.close()

#-----------------------------------------------------------------------------
#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        global Go
        Go = 0
        self.master = master
        master.title('Orphan Texture finder')

        self.NameEntry = Entry(self.master,width=30)
        self.NameEntry.grid(row=1,column=1)
        self.NameEntry.insert(END,'Hit Go to start')
        self.NameEntry.clipboard_clear()

        # Go/nogo buttons.
        self.buttonGo = Button(self.master, text='Go', command=self.handleGo)
        self.buttonGo.grid(row=3, column=0)

        self.buttonCancel = Button(self.master, text='Cancel', command=self.handleCancel)
        self.buttonCancel.grid(row=3, column=2)

    # - - - - - - - - - - - - - - - - - -

    def LocateFiles(self):
        # Main loop is a path.walk.  The inner function of the walk
        # reads the CR2, finds each figureResFile, and appends the
        # basename to the UsedTexs dict.

        os.path.walk(SourcePath,ReadSourceInDir,'.CR2')
        os.path.walk(SourcePath,ReadSourceInDir,'.PP2')
        os.path.walk(SourcePath,ReadSourceInDir,'.HR2')
        os.path.walk(SourcePath,ReadSourceInDir,'.PZ2')
        os.path.walk(SourcePath,ReadSourceInDir,'.FC2')
        os.path.walk(SourcePath,ReadSourceInDir,'.CM2')
        # We now have a list of USED objects.

        # Walk through Textures.  For each tex we
        # find, get its basename and see if it's in the UsedTexs dict.
        # If not there, write its complete path to the log file.

        os.path.walk(BasePath,ReadTexsInDir,None)
        WriteLog()

    # - - - - - - - - - - - - - - - - - -

    def handleGo(self):
        ShowStatus("Go drink some coffee, I'm working!.....")
        self.LocateFiles()
        ShowStatus("Done!")
        self.master.destroy()
        self.master.quit()

    # - - - - - - - - - - - - - - - - - -

    def handleCancel(self):
        self.master.destroy()
        self.master.quit()

#------------------------------------------------------------
#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

app = App(root, '')
root.mainloop()


#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
